
//	K1-16/16 microcode
//	2010-04-06
//	2015-12-07


//	FAILURES 2010-04-06:
//	0MHz:	ld sr,sr,cy: oft
//	2MHz:	all tests passed
//	4MHz:	all tests passed
//	8MHz:	ld sr,sr,cy: 4
//	16MHz:	Z-Test: 9



// ERROR: 	mca: Redefinition lokaler Label wird nicht bemerkt!




// ================================================================
//	Options and Symbolic Names:
// ================================================================

// 	Welcher Quartz-Vorteiler soll maximal eingestellt werden?
//	 2 -> xtal/2  = 16 MHz
//	 4 -> xtal/4  =  8 MHz
//	 8 -> xtal/8  =  4 MHz
//	16 -> xtal/16 =  2 MHz
//
xtal:	=	32000					// kHz
clock_predivider: = 2				// 2/4/8/16: Der Assembler kennt 'clock_predivider' und ersetzt ggf. "set clock2" und "4".
//clock	=	xtal/clock_predivider	// FYI
fast_ival_z_test: = 0				// does "tst,nnnn : z" work? (req. 74AC151 on SSW)
									// note: "tst (areg)" does not work, even with 74AC151!


#include "defs.asm"
#include "macros.asm"
//#include "millicode.asm"
//#include "millicode_int.asm"



// ========================================================================
// Global Data
// start at $0000
// =======================================================================


// Interrupts:

int_vector			data 16		// interrupt vectors
int_mask			data 1		// enable mask for installed handlers

// Memory Management:

gdata_ptr			data 1		// allocation pointer for zeropage global data
gcode_ptr			data 1		// start of space for new words
hp_base				data 1		// bottom of data heap
sp_base				data 1		// top of return stack
mem_data_start		data 1		// Start of dynamic data blocks
mem_data_end		data 1		// Start of free memory between data and pointers
mem_ptr_start		data 1		// Start of global data pointers
mem_free_ptr		data 1		// Pointer to linked list of free global pointers
mem_end				data 1
mem_ptr_end	:= mem_end			// End of local data pointers

// Misc:

opcode_Ret			data 1		// opcode Ret: return_to_millicode




// ======================================================================
// 			error exit
// ======================================================================

// 	LED Codes:		(RYG = red, yellow, green)
//
//	---		after start
//	--G		after first pass OK
//	---		after second pass OK	=> blinks with ~ 0.5 … 1Hz
//	R--		error: a test failed

:error 
		clr	clock4
		clr	clock2
		set	led_red
		clr	led_yel
		clr	led_grn
l1		tst	atmp			// show error address on dbus
		jp	l1

	instr ERROR
		ld atmp,$
		jp	error
	end

	instr TRAP(COND)
		if COND
			ld atmp,$
			jp	error
		then
	end


// ================================================================
//	RESET
//
//	Reset entry condition:
//	• $3FFF:1 with 'cmddis=1' and 'cond=0' (for next instr) forced by INIT
//	• $0000:0 is executed
// ================================================================

: reset_7fff = $3FFF:1
		clr	reset			// 'nop'

: reset = $0000:0
		clr halt
		di

		set	clock4			// set full speed …
		set	clock2			// … except if xtal/clock > 2

		clr led_red			// LEDs indicate: started
		clr led_yel
		clr led_grn


// ======================================
// 	CPU & RAM Test
// ======================================

start:

#include "cmd.asm"
#include "flags.asm"
#include "alu.asm"
#include "dregs.asm"		
#include "aregs.asm"
#include "ramtest.asm"
//#include "cmd_lo.asm"

		set led_grn			// 1st half of test loop passed
		jp	again

: again
#include "cmd.asm"	

#include "flags.asm"
		nop
#include "flags.asm"
		nop
		nop
#include "flags.asm"
		nop
		nop
		nop
#include "flags.asm"

#include "alu.asm"
#include "dregs.asm"		
#include "aregs.asm"
#include "ramtest.asm"
//#include "cmd_lo.asm"

		clr led_grn			// 2nd half of test loop passed
		jp	start			// again…





// #include	"ldir.asm"
// TODO:
// IO
// Interrupt



















